//===----------------------------------------------------------------------===//
//
// Copyright (c) 2016 The University of Utah
// All rights reserved.
//
// This file is distributed under the University of Illinois Open Source
// License.  See the file COPYING for details.
//
//===----------------------------------------------------------------------===//

#ifndef GIT_VERSION_H
#define GIT_VERSION_H

extern const char git_version[];

#endif // GIT_VERSION_H
